/*
 * Turbo Programmer Utilities, turbo-prog-utils, www.bladox.com 
 *
 * Copyright (C) 2004 BLADOX, s.r.o.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <tfslib.h>
#include <tproglib.h>

#include <stdlib.h>
#include <string.h>

#include <stdio.h>
#include <unistd.h>

#include "../module-src/df.h"

void help ()
{
  printf ("Dump block\n");
  printf ("Usage: [args] block_in_hex\n");
  printf ("-d device\tuse USB device\n");
  printf ("-h\thelp\n");
  printf ("-v\tverbose\n");
  exit (-1);
}

int main (int argc, char **argv)
{
  u32 i;
  u16 c;
  u8 nb;
  u16 pos;
  u8 buf[1000];
  u8 len;
  int result;
  u8 sort;
  u16 j;
  u8 *d_name;
  u8 path[1000] = { '/', '\0' };
  u8 name[1000];
  u16 name_end;
  u16 path_end;
  b32 f;
  u8 dump = 0;
  Inode ino;
  u8 off;
  u32 page;
  u8 *dev = USBDEVICE;

  do
  {
    result = getopt (argc, argv, "hd:v");
    switch (result)
    {
      case 'd':
	dev = optarg;
	break;
      case 'h':
	help ();
	break;
      case 'v':
	verbose_inc ();
	break;
    }
  }
  while (result > 0);

  if (argc - optind != 1)
    help ();
  sscanf (argv[optind], "%x", &i);

  open_device (dev);
  if (select_app (APP_DF) != NO_ERROR)
  {
    close_device ();
    fprintf (stderr, "Cannot open application %s\n", APP_DF);
    exit (-1);
  }

  usb_send (ACTION_DF_INIT, 0, NULL, NULL, NULL);

  if (mc_detect () == MC_OK)
  {
    u8 name[100];

    printf ("Memory card detected\n");
    mc_get_label (name);
    printf ("Card label: %s\n\n", name);
  }
  else if (mc_detect () == MC_NOT_PRESENT)
  {
    printf ("Memory card NOT PRESENT\n");
    close_device ();
    exit (1);
  }

  off = 1;
  if (mc_blocks () == 4)
    off = 2;
  page = i >> off;
  printf ("BLOCK %04X PAGE %04X\n", i, page);

  dataFlashMemoryToBufferRead (0, page);
  dataFlashBufferRead (0, buf, (i & (mc_blocks () - 1)) * 264, 264);

  for (i = 0; i < 17; i++)
  {
    printf ("%03X:\t", i * 16);
    for (j = 0; j < 16; j++)
    {
      if (i * 16 + j < 264)
	printf ("%02X ", buf[i * 16 + j]);
    }
    if (i != 16)
    {
      printf ("   ");
      for (j = 0; j < 16; j++)
      {
	c = '.';
	if (i * 16 + j < 264)
	{
	  if (buf[i * 16 + j] >= 0x20 && buf[i * 16 + j] < 0x7f)
	    c = buf[i * 16 + j];
	  printf ("%c", c);
	}
      }
    }
    printf ("\n");
  }

  close_device ();
}
